/* File giving restriction message, if restrictions are enforced */
#define RESTRICT_FILE OWNDIR"/lib/archive/files/loadcheck"

/* List of directories to search for the games */
char *dir_list[] = { OWNDIR"/src/", "END" };

#define NO_GAME -2
#define OK_GAME 1
#define GAME_LOG OWNDIR"/lib/archive/files/GAMELOG"
